// file PersistentData.java
// jlouie 1.98
// this class is a container for application data objects
// supports persistence and hides the method of persistence
// implements a backup file strategy to recover from failed write on exit.

import java.io.*;

public class PersistentData {

    private static final String fileName="MCalc.dat";
    private static final String backupName="MCalc.bak";
    
    Model model;
    File dat= new File(fileName);
    File bak= new File(backupName);

    // get persistent object if it exists
    // if not exists create new object
    public PersistentData() {
        boolean isValidFile= false;
        // load any existing dat file
        isValidFile= isLoadModel(dat);
        // if none exist load any bak file
        if (! isValidFile){isValidFile= isLoadModel(bak);}
        // if no file exist, create new object
        if (! isValidFile){model= new Model();}
    }

    public Model getModel() {
        return model;
    }
    
    private boolean isLoadModel(File file) {
        model= readFile(file);
        if (model != null) return true;
        else return false;
    }

    // save model object here
    // implementation is hidden
    // we use object persistence here
    // we could flatten object and write to a relational database
    // or we could store object as an ASCII file
    public void saveModel() {
        if (dat.exists()) {
                if (bak.exists()) {bak.delete();}
                if (! dat.renameTo(bak)){
                    System.out.println("Unable to rename dat file!");
                }
        }
        // now we should have a copy of any old data as bak file
        writeFile(dat, model);
    }
    
    private Model readFile(File source){
        FileInputStream fis= null;
        ObjectInputStream ois= null;
        Model temp= null;
        
        try {
            fis = new FileInputStream(source);
            ois= new ObjectInputStream(fis);
            temp= (Model)ois.readObject();
        }
        catch(ClassNotFoundException e){
            System.out.println(e);
        }
        catch(IOException e2) {
            System.out.println("File named "+e2.getMessage()+" not found.");
        }
        finally {
            if (ois != null) {
                try{ois.close();}
                catch(Exception ignored) {;}
            }
            return temp;
       }    
    }
    
    private void writeFile(File destination, Model model) {
        FileOutputStream fos= null;
        ObjectOutputStream oos= null;
        try {
            fos= new FileOutputStream(destination);
            oos= new ObjectOutputStream(fos);
            oos.writeObject(model);
            oos.flush();
        }
        catch(IOException e) {
            System.out.println(e); 
        }
        finally {
            if (oos != null) {
                try {oos.close();}
                catch(IOException ignored) {;}
            }
            //try{System.in.read();}catch(Exception ignored) {;}
        }   
    }
}